<?php
/**
 * WooCommerce Tab Manager
 *
 * This source file is subject to the GNU General Public License v3.0
 * that is bundled with this package in the file license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@skyverge.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade WooCommerce Tab Manager to newer
 * versions in the future. If you wish to customize WooCommerce Tab Manager for your
 * needs please refer to http://docs.woocommerce.com/document/tab-manager/
 *
 * @author      SkyVerge
 * @copyright   Copyright (c) 2012-2024, SkyVerge, Inc. (info@skyverge.com)
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

namespace SkyVerge\WooCommerce\Tab_Manager;

defined('ABSPATH') or exit;

use \SkyVerge\WooCommerce\PluginFramework\v5_11_0 as Framework;
use WC_Product;
use WC_Tab_Manager;
use WP_REST_Request;

/**
 * REST API handler.
 *
 * @since 1.16.1
 */
class RestApi
{
	protected WC_Tab_Manager $plugin;

	public function __construct(WC_Tab_Manager $plugin)
	{
		$this->plugin = $plugin;

		$this->addHooks();
	}

	protected function addHooks() : void
	{
		add_filter('woocommerce_rest_pre_insert_product_object', [$this, 'processProductTabs'], 20, 3);
	}

	/**
	 * Process custom tabs for the given product created/updated via REST API
	 */
	public function processProductTabs(WC_Product $product, WP_REST_Request $request, bool $creating) : WC_Product
	{
		if (! function_exists('wc_tab_manager_process_tabs')) {
			include_once $this->plugin->get_plugin_path().'/admin/woocommerce-tab-manager-admin-functions.php';
		}

		if ($tabLayoutOverrideInRequest = isset($request['_override_tab_layout'])) {
			$overrideLayout = $request['_override_tab_layout'];
		} else {
			// pull the existing value, to use for reference further down
			// this ensures we don't change the value if it wasn't supplied in the request
			$overrideLayout = $product->get_meta('_override_tab_layout', true, 'edit');
		}

		$overrideLayout = in_array($overrideLayout, [true, 'yes'], true);

		$tabs = wc_tab_manager_process_tabs($product->get_id(), $request);

		if ($creating && ! $tabs) {
			// fallback to the configured default tabs layout
			$tabs = $this->plugin->getDefaultTabsLayout();
		}

		if (! $tabs && ! $tabLayoutOverrideInRequest) {
			// skip early if no tabs data was provided in the request
			return $product;
		}

		return $this->storeProductTabs($product, $tabs, $overrideLayout, ! $creating);
	}

	protected function storeProductTabs(WC_Product $product, array $tabs, bool $overrideLayout, bool $update = false) : WC_Product
	{
		if ($tabs) {
			$oldTabs = $update ? $product->get_meta('_product_tabs') : [];

			$product->update_meta_data('_product_tabs', $tabs);

			do_action('wc_tab_manager_product_tabs_updated', $tabs, is_array($oldTabs) ? $oldTabs : []);
		}

		$product->update_meta_data('_override_tab_layout', $overrideLayout ? 'yes' : 'no');

		$this->updateProductSearchData($product->get_id(), $tabs, $overrideLayout);

		return $product;
	}

	protected function updateProductSearchData(int $productId, array $tabs, bool $overrideLayout) : void
	{
		// Extract product & global tab IDs from tab data array.
		$tabIds = array_filter(array_map(static function ($tab) {
			return 'product' === $tab['type'] || 'global' === $tab['type'] ? $tab['id'] : false;
		}, $tabs));

		if (! $tabIds) {
			// Skip if we do not have any tabs to process.
			return;
		}

		$this->plugin->get_search_instance()->update_products_for_tabs($tabIds, [
			'product_id' => $productId,
			'action'     => $overrideLayout ? 'update' : 'remove',
		]);
	}
}
